import unittest
from .base import Base
from pyramid.path import DottedNameResolver


class TestExtensions(Base, unittest.TestCase):

    def test_custom_extension(self):
        from pyramid_jinja2 import create_environment_from_options
        from pyramid_jinja2.settings import parse_env_options_from_settings

        options = {
            'extensions': 'pyramid_jinja2.tests.extensions.TestExtension',
        }
        settings = parse_env_options_from_settings(
            options, '', maybe_dotted, None)
        env = create_environment_from_options(settings, {})
        ext = env.extensions[
            'pyramid_jinja2.tests.extensions.TestExtension']
        import pyramid_jinja2.tests.extensions
        self.assertEqual(ext.__class__,
                         pyramid_jinja2.tests.extensions.TestExtension)

    def test_i18n(self):
        from pyramid_jinja2 import create_environment_from_options
        from pyramid_jinja2.settings import parse_env_options_from_settings

        settings = parse_env_options_from_settings({}, '', maybe_dotted, None)
        env = create_environment_from_options(settings, {})

        self.assertTrue(hasattr(env, 'install_gettext_translations'))

        self.config.add_translation_dirs('pyramid_jinja2.tests:locale/')
        self.request.locale_name = 'en'
        template = env.get_template(
            'pyramid_jinja2.tests:templates/i18n.jinja2')
        self.assertEqual(template.render(),
                         'some untranslated text here\nyay it worked!')


resolver = DottedNameResolver()
maybe_dotted = resolver.maybe_resolve


class GetTextWrapperTests(unittest.TestCase):

    def test_it(self):
        from pyramid_jinja2.i18n import GetTextWrapper

        class MyGetTextWrapper(GetTextWrapper):
            class localizer:
                @staticmethod
                def translate(s, domain):
                    return s

                @staticmethod
                def pluralize(s1, s2, n, domain):
                    return s2

            def __init__(self):
                GetTextWrapper.__init__(self, 'messages')

        self.assertEqual(MyGetTextWrapper().gettext('foo'), 'foo')
        self.assertEqual(MyGetTextWrapper().ngettext('foo', 'foos', 3), 'foos')



from pyramid.url import resource_url, route_url, static_url
from pyramid.url import route_path, static_path
from pyramid.threadlocal import get_current_request
from jinja2 import contextfilter


__all__ = [
    'resource_url_filter',
    'model_url_filter',
    'route_url_filter',
    'route_path_filter',
    'static_url_filter',
    'static_path_filter',
]


@contextfilter
def resource_url_filter(ctx, model, *elements, **kw):
    """A filter from ``model`` to a string representing the absolute URL.
    This filter calls :py:func:`pyramid.url.resource_url`.

    Example::

        <a href="{{'my_traversable_object'|resource_url}}">
            See my object
        </a>

    You can also specify optional view name attached at the end of a path::

        <a href="{{'my_traversable_object'|resource_url('edit')}}">
            Edit my object
        </a>

    """
    request = ctx.get('request') or get_current_request()
    return resource_url(model, request, *elements, **kw)


@contextfilter
def model_url_filter(ctx, model, *elements, **kw):
    """A filter from ``model`` to a string representing the absolute URL.
    This filter calls :py:func:`pyramid.url.resource_url`.

    .. note ::

        This is being deprecated.
        See :py:func:`pyramid_jinja2.filters.resource_url`

    """
    request = ctx.get('request') or get_current_request()
    return resource_url(model, request, *elements, **kw)


@contextfilter
def model_path_filter(ctx, model, *elements, **kw):
    """A filter from ``model`` to a string representing the relative URL.
    This filter calls :py:meth:`pyramid.request.Request.resource_path`.
    """
    request = ctx.get('request') or get_current_request()
    return request.resource_path(model, *elements, **kw)


@contextfilter
def route_url_filter(ctx, route_name, *elements, **kw):
    """A filter from ``route_name`` to a string representing the absolute URL.
    This filter calls :py:func:`pyramid.url.route_url`.

    Example::

        <a href="{{'login'|route_url}}">
            Sign in
        </a>

    """
    request = ctx.get('request') or get_current_request()
    return route_url(route_name, request, *elements, **kw)


@contextfilter
def route_path_filter(ctx, route_name, *elements, **kw):
    """A filter from ``route_name`` to a string representing the relative URL.
    This filter calls :py:func:`pyramid.url.route_path`.
    """
    request = ctx.get('request') or get_current_request()
    return route_path(route_name, request, *elements, **kw)


@contextfilter
def static_url_filter(ctx, path, **kw):
    """A filter from ``path`` to a string representing the absolute URL.
    This filter calls :py:func:`pyramid.url.static_url`.

    Example::

       <link rel="stylesheet" href="{{'yourapp:static/css/style.css'|static_url}}" />

    """
    request = ctx.get('request') or get_current_request()
    return static_url(path, request, **kw)


@contextfilter
def static_path_filter(ctx, path, **kw):
    """A filter from ``path`` to a string representing the relative URL.
    This filter calls :py:func:`pyramid.url.static_path`.
    """
    request = ctx.get('request') or get_current_request()
    return static_path(path, request, **kw)



from pyramid import i18n
from pyramid.threadlocal import get_current_request


class GetTextWrapper(object):
    """Implements `gettext` and `ngettext` functions for
    :meth:`jinja2.Environment.install_gettext_translations`
    """

    def __init__(self, domain):
        self.domain = domain

    @property
    def localizer(self):
        request = get_current_request()
        try:
            return request.localizer
        except AttributeError: # pragma: nocover (pyramid < 1.5)
            return i18n.get_localizer(request)

    def gettext(self, message):
        """Implements jinja.ext.i18n `gettext` function"""
        return self.localizer.translate(message,
                                        domain=self.domain)

    def ngettext(self, singular, plural, n):
        """Implements jinja.ext.i18n `ngettext` function"""
        return self.localizer.pluralize(singular, plural, n,
                                        domain=self.domain)
